@@ -108,8 +108,9 @@ class ClipboardHandler:
                     return
         if not os.path.isdir(cons.TMP_FOLDER): os.mkdir(cons.TMP_FOLDER)
         html_text = self.dad.html_handler.selection_export_to_html(text_buffer, iter_sel_start, iter_sel_end, self.dad.syntax_highlighting)
-        if self.dad.syntax_highlighting == cons.CUSTOM_COLORS_ID:
-            plain_text = text_buffer.get_text(iter_sel_start, iter_sel_end)
+        if self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
+            txt_handler = exports.Export2Txt(self.dad)
+            plain_text = txt_handler.node_export_to_txt(text_buffer, "", sel_range=[iter_sel_start.get_offset(), iter_sel_end.get_offset()])
             rich_text = self.rich_text_get_from_text_buffer_selection(text_buffer, iter_sel_start, iter_sel_end)
             self.clipboard.set_with_data([(t, 0, 0) for t in (TARGET_CTD_PLAIN_TEXT, TARGET_CTD_RICH_TEXT, TARGETS_HTML[0])],
                                          self.get_func,
@@ -130,11 +131,11 @@ class ClipboardHandler:
         elif target == TARGETS_HTML[0]: selectiondata.set('UTF8_STRING', 8, data[2])
         elif target == TARGET_CTD_CODEBOX:
             dom = xml.dom.minidom.Document()
-            self.dad.xml_handler.codebox_element_to_xml([0, data[0], cons.TAG_PROP_LEFT], dom)
+            self.dad.xml_handler.codebox_element_to_xml([0, data[0], cons.TAG_PROP_LEFT], dom, dom)
             selectiondata.set('UTF8_STRING', 8, dom.toxml())
         elif target == TARGET_CTD_TABLE:
             dom = xml.dom.minidom.Document()
-            self.dad.xml_handler.table_element_to_xml([0, data[0], cons.TAG_PROP_LEFT], dom)
+            self.dad.xml_handler.table_element_to_xml([0, data[0], cons.TAG_PROP_LEFT], dom, dom)
             selectiondata.set('UTF8_STRING', 8, dom.toxml())
         elif target in TARGETS_IMAGES: selectiondata.set_pixbuf(data)
 
@@ -157,24 +158,24 @@ class ClipboardHandler:
                     self.clipboard.request_contents(target, self.to_plain_text)
                     return
         #print targets
-        if TARGET_CTD_RICH_TEXT in targets and self.dad.syntax_highlighting == cons.CUSTOM_COLORS_ID:
+        if TARGET_CTD_RICH_TEXT in targets and self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
             self.clipboard.request_contents(TARGET_CTD_RICH_TEXT, self.to_rich_text)
             return
-        if TARGET_CTD_CODEBOX in targets and self.dad.syntax_highlighting == cons.CUSTOM_COLORS_ID:
+        if TARGET_CTD_CODEBOX in targets and self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
             self.clipboard.request_contents(TARGET_CTD_CODEBOX, self.to_codebox)
             return
-        if TARGET_CTD_TABLE in targets and self.dad.syntax_highlighting == cons.CUSTOM_COLORS_ID:
+        if TARGET_CTD_TABLE in targets and self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
             self.clipboard.request_contents(TARGET_CTD_TABLE, self.to_table, None)
             return
         for target in TARGETS_HTML:
-            if target in targets and self.dad.syntax_highlighting == cons.CUSTOM_COLORS_ID:
+            if target in targets and self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
                 self.clipboard.request_contents(target, self.to_html)
                 return
         if TARGET_URI_LIST in targets:
             self.clipboard.request_contents(TARGET_URI_LIST, self.to_uri_list)
             return
         for target in TARGETS_IMAGES:
-            if target in targets and self.dad.syntax_highlighting == cons.CUSTOM_COLORS_ID:
+            if target in targets and self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
                 self.clipboard.request_contents(target, self.to_image)
                 return
         for target in TARGETS_PLAIN_TEXT:
@@ -189,7 +190,7 @@ class ClipboardHandler:
     def to_uri_list(self, clipboard, selectiondata, data):
         """From Clipboard to URI list"""
         selection_data = re.sub(cons.BAD_CHARS, "", selectiondata.data)
-        if self.dad.syntax_highlighting != cons.CUSTOM_COLORS_ID:
+        if self.dad.syntax_highlighting != cons.RICH_TEXT_ID:
             iter_insert = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert())
             self.dad.curr_buffer.insert(iter_insert, selection_data)
             return
@@ -260,7 +261,7 @@ class ClipboardHandler:
         #print "###########################"
         #for char in selection_data: print ord(char)
         selection_data = re.sub(cons.BAD_CHARS, "", selection_data)
-        html_import = imports.HTMLFromClipboardHandler(self.dad)
+        html_import = imports.HTMLHandler(self.dad)
         xml_string = html_import.get_clipboard_selection_xml(selection_data)
         self.from_xml_string_to_buffer(xml_string)
 
@@ -273,7 +274,7 @@ class ClipboardHandler:
         iter_insert = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert())
         start_offset = iter_insert.get_offset()
         self.dad.curr_buffer.insert(iter_insert, plain_text)
-        if self.dad.syntax_highlighting == cons.CUSTOM_COLORS_ID:
+        if self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
             web_links_offsets = imports.get_web_links_offsets_from_plain_text(plain_text)
             if web_links_offsets:
                 for offsets in web_links_offsets:
@@ -351,6 +352,7 @@ class ClipboardHandler:
     def to_image(self, clipboard, selectiondata, data):
         """From Clipboard to Image"""
         pixbuf = selectiondata.get_pixbuf()
+        pixbuf.link = ""
         self.dad.image_insert(self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert()), pixbuf)
 
     def dom_node_to_image(self, dom_node):
@@ -358,15 +360,19 @@ class ClipboardHandler:
         if dom_node.hasAttribute("justification"): justification = dom_node.attributes["justification"].value
         else: justification = cons.TAG_PROP_LEFT
         if dom_node.hasAttribute("anchor"):
-            pixbuf = gtk.gdk.pixbuf_new_from_file(cons.ANCHOR_CHAR)
-            pixbuf = pixbuf.scale_simple(self.dad.anchor_size, self.dad.anchor_size, gtk.gdk.INTERP_BILINEAR)
+            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(cons.ANCHOR_CHAR, self.dad.anchor_size, self.dad.anchor_size)
             pixbuf.anchor = dom_node.attributes["anchor"].value
+        elif dom_node.hasAttribute("filename"):
+            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(cons.FILE_CHAR, self.dad.embfile_size, self.dad.embfile_size)
+            pixbuf.filename = dom_node.attributes["filename"].value
+            pixbuf.embfile = base64.b64decode(dom_node.firstChild.data)
         else: pixbuf = machines.get_pixbuf_from_encoded_buffer(dom_node.firstChild.data)
         if pixbuf:
+            pixbuf.link = dom_node.attributes["link"].value if dom_node.hasAttribute("link") else ""
             self.dad.image_insert(self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert()),
-                                  pixbuf,
-                                  image_justification=justification,
-                                  text_buffer=self.dad.curr_buffer)
+                pixbuf,
+                image_justification=justification,
+                text_buffer=self.dad.curr_buffer)
 
     def to_codebox(self, clipboard, selectiondata, data):
         """From Clipboard to CodeBox"""
@@ -500,5 +506,5 @@ class ClipboardHandler:
                                                     change_case=change_case, dom=dom)
         if obj_element:
             if obj_element[0] == "pixbuf": self.dad.xml_handler.pixbuf_element_to_xml(obj_element[1], dom_iter, dom)
-            elif obj_element[0] == "table": self.dad.xml_handler.table_element_to_xml(obj_element[1], dom_iter)
-            elif obj_element[0] == "codebox": self.dad.xml_handler.codebox_element_to_xml(obj_element[1], dom_iter)
+            elif obj_element[0] == "table": self.dad.xml_handler.table_element_to_xml(obj_element[1], dom_iter, dom)
+            elif obj_element[0] == "codebox": self.dad.xml_handler.codebox_element_to_xml(obj_element[1], dom_iter, dom)
