@@ -22,12 +22,12 @@
 import os, sys
 
 
-VERSION = "0.32.0"
+VERSION = "0.34.0"
 APP_NAME = "cherrytree"
 NEWER_VERSION_URL = "http://www.giuspen.com/software/version_cherrytree"
 if sys.platform.startswith("win"):
     IS_WIN_OS = True
-    CONFIG_DIR = os.path.join(os.environ['APPDATA'], APP_NAME)
+    SZA_PATH = os.path.join(SHARE_PATH, "7za.exe")
     if SHARE_PATH:
         EXE_DIR = SHARE_PATH
         os.chdir(EXE_DIR)
@@ -36,8 +36,12 @@ if sys.platform.startswith("win"):
     GLADE_PATH = os.path.join(EXE_DIR, 'glade/')
     SPECS_PATH = os.path.join(EXE_DIR, 'language-specs/')
     LOCALE_PATH = os.path.join(EXE_DIR, 'locale/')
+    if not os.path.isfile(os.path.join(EXE_DIR, 'config.cfg')):
+        CONFIG_DIR = os.path.join(os.environ['APPDATA'], APP_NAME)
+    else: CONFIG_DIR = EXE_DIR
 else:
     IS_WIN_OS = False
+    SZA_PATH = "7za"
     CONFIG_DIR = os.path.join(os.path.expanduser('~'), '.config/cherrytree')
     TMP_FOLDER = '/tmp/ct_tmp/'
     if not SHARE_PATH:
@@ -59,7 +63,7 @@ except: HAS_APPINDICATOR = False
 XDG_CURRENT_DESKTOP = 'XDG_CURRENT_DESKTOP'
 HAS_SYSTRAY = not (XDG_CURRENT_DESKTOP in os.environ and os.environ[XDG_CURRENT_DESKTOP] == "Unity")
 
-AVAILABLE_LANGS = ['default', 'cs', 'de', 'en', 'es', 'fr', 'it', 'nl', 'pl', 'pt_BR', 'ru', 'uk', 'zh_CN']
+AVAILABLE_LANGS = ['default', 'cs', 'de', 'en', 'es', 'fr', 'hy', 'it', 'nl', 'pl', 'pt_BR', 'ru', 'uk', 'zh_CN']
 COLOR_48_LINK_WEBS = "#00004444ffff"
 COLOR_48_LINK_NODE = "#071c838e071c"
 COLOR_48_LINK_FILE = "#8b8b69691414"
@@ -114,9 +118,11 @@ LINK_TYPE_WEBS = "webs"
 LINK_TYPE_FILE = "file"
 LINK_TYPE_FOLD = "fold"
 LINK_TYPE_NODE = "node"
-CUSTOM_COLORS_ID = "custom-colors"
+RICH_TEXT_ID = "custom-colors"
+PLAIN_TEXT_ID = "plain-text"
 STYLE_SCHEME_DEFAULT = "cobalt"
 ANCHOR_CHAR = GLADE_PATH + 'anchor.png'
+FILE_CHAR = GLADE_PATH + 'file_icon.png'
 NODES_ICONS = {0:'cherry_red', 1:'cherry_blue', 2:'cherry_orange', 3:'cherry_cyan',
                4:'cherry_orange_dark', 5:'cherry_sherbert', 6:'cherry_yellow'}
 CODE_ICONS = {"python": 'cherry_green', "sh":'cherry_purple',
@@ -154,13 +160,16 @@ CHAR_LESSER = "<"
 CHAR_GREATER = ">"
 CHAR_STAR = "*"
 CHAR_QUESTION = "?"
+CHAR_COMMA = ","
 CHAR_COLON = ":"
+CHAR_SEMICOLON = ";"
 CHAR_USCORE = "_"
 CHAR_EQUAL = "="
 CHAR_BR_OPEN = "{"
 CHAR_BR_CLOSE = "}"
 CHAR_CARET = "^"
 CHAR_PIPE = "|"
+CHAR_AMPERSAND = "&"
 
 SPECIAL_CHAR_ARROW_RIGHT = "→"
 SPECIAL_CHAR_ARROW_LEFT = "←"
@@ -199,10 +208,10 @@ STOCKS_N_FILES = [
 'format_text_latest.png', 'format_text_clear.png', 'format_text.png',
 'object-rotate-left.png', 'object-rotate-right.png',
 'to_pdf.png', 'to_txt.png', 'to_html.png', 'to_cherrytree.png', 'export_from_cherrytree.png', STR_STOCK_CT_IMP + '.png',
-'from_cherrytree.png', 'from_txt.png', 'cherrytree.png', 'quit-app.png',
+'from_cherrytree.png', 'from_txt.png', 'from_html.png', 'cherrytree.png', 'quit-app.png',
 'new-instance.png', 'toolbar.png', 'cherries.png', 'tree-node-add.png',
 'tree-subnode-add.png', 'help-contents.png', 'index.png', 'timestamp.png',
-'calendar.png', 'horizontal_rule.png']
+'calendar.png', 'horizontal_rule.png', 'file_icon.png']
 
 NODES_STOCKS = ['node_bullet', 'node_no_icon', 'cherry_black',
                 'cherry_blue', 'cherry_cyan', 'cherry_green',
@@ -251,7 +260,9 @@ node_id INTEGER,
 offset INTEGER,
 justification TEXT,
 anchor TEXT,
-png BLOB
+png BLOB,
+filename TEXT,
+link TEXT
 )"""
 
 TABLE_CHILDREN_CREATE = """CREATE TABLE children (
@@ -291,6 +302,7 @@ UI_INFO = """
             <menuitem action='HandleImage'/>
             <menuitem action='HandleTable'/>
             <menuitem action='HandleCodeBox'/>
+            <menuitem action='EmbFileInsert'/>
             <menuitem action='HandleLink'/>
             <menuitem action='HandleAnchor'/>
             <menuitem action='InsertTOC'/>
@@ -303,6 +315,8 @@ UI_INFO = """
                 <menuitem action='ToggleCase'/>
             </menu>
             <separator/>
+            <menuitem action='EnaDisSpellCheck'/>
+            <separator/>
             <menuitem action='PastePlainText'/>
             <separator/>
             <menuitem action='CutRow'/>
@@ -346,6 +360,7 @@ UI_INFO = """
         <menu action='SearchMenu'>
             <menuitem action='FindInNode'/>
             <menuitem action='FindInNodes'/>
+            <menuitem action='FindInSelNSub'/>
             <menuitem action='FindNode'/>
             <menuitem action='FindAgain'/>
             <menuitem action='FindBack'/>
@@ -354,6 +369,7 @@ UI_INFO = """
         <menu action='ReplaceMenu'>
             <menuitem action='ReplaceInNode'/>
             <menuitem action='ReplaceInNodes'/>
+            <menuitem action='ReplaceInSelNSub'/>
             <menuitem action='ReplaceInNodesNames'/>
             <menuitem action='ReplaceAgain'/>
         </menu>
@@ -384,7 +400,10 @@ UI_INFO = """
             <menuitem action='FromCherryTree'/>
             <menuitem action='FromTxtFile'/>
             <menuitem action='FromTxtFolder'/>
+            <menuitem action='FromHtmlFile'/>
+            <menuitem action='FromHtmlFolder'/>
             <menuitem action='FromBasket'/>
+            <menuitem action='FromEPIMHTML'/>
             <menuitem action='FromGnote'/>
             <menuitem action='FromKeepNote'/>
             <menuitem action='FromKeyNote'/>
@@ -434,6 +453,7 @@ UI_INFO = """
         <toolitem action='HandleImage'/>
         <toolitem action='HandleTable'/>
         <toolitem action='HandleCodeBox'/>
+        <toolitem action='EmbFileInsert'/>
         <toolitem action='HandleLink'/>
         <toolitem action='HandleAnchor'/>
         <separator/>
@@ -466,30 +486,17 @@ UI_INFO = """
         <separator/>
         <menuitem action='EditImage'/>
         <menuitem action='SaveImage'/>
+        <separator/>
+        <menuitem action='EditImageLink'/>
+        <menuitem action='DismissImageLink'/>
     </popup>
 
     <popup name='AnchorMenu'>
         <menuitem action='EditAnchor'/>
     </popup>
 
-    <popup name='TableMenu'>
-        <menuitem action='CutTable'/>
-        <menuitem action='CopyTable'/>
-        <menuitem action='DeleteTable'/>
-        <separator/>
-        <menuitem action='TableRowAdd'/>
-        <menuitem action='TableRowCut'/>
-        <menuitem action='TableRowCopy'/>
-        <menuitem action='TableRowPaste'/>
-        <menuitem action='TableRowDelete'/>
-        <separator/>
-        <menuitem action='TableRowUp'/>
-        <menuitem action='TableRowDown'/>
-        <menuitem action='TableSortRowsDesc'/>
-        <menuitem action='TableSortRowsAsc'/>
-        <separator/>
-        <menuitem action='TableEditProp'/>
-        <menuitem action='TableExport'/>
+    <popup name='EmbFileMenu'>
+        <menuitem action='EmbFileSave'/>
     </popup>
 </ui>
 """
@@ -529,25 +536,26 @@ def get_entries(inst):
     ( "ExitApp", "quit-app", _("_Exit CherryTree"), "<control><shift>Q", _("Exit from CherryTree"), inst.quit_application_totally),
     ( "Preferences", "gtk-preferences", _("_Preferences"), "<control><alt>P", _("Preferences"), inst.dialog_preferences),
     ( "HorizontalRule", "horizontal_rule", _("Insert _Horizontal Rule"), "<control>R", _("Insert Horizontal Rule"), inst.horizontal_rule_insert),
-    ( "Timestamp", "timestamp", _("Insert Ti_mestamp"), "<control>M", _("Insert Timestamp"), inst.timestamp_insert),
-    ( "BulletedList", "list_bulleted", _("Set/Unset _Bulleted List"), None, _("Set/Unset the Current Paragraph/Selection as a Bulleted List"), inst.list_bulleted_handler),
-    ( "NumberedList", "list_numbered", _("Set/Unset _Numbered List"), None, _("Set/Unset the Current Paragraph/Selection as a Numbered List"), inst.list_numbered_handler),
-    ( "ToDoList", "list_todo", _("Set/Unset _To-Do List"), None, _("Set/Unset the Current Paragraph/Selection as a To-Do List"), inst.list_todo_handler),
+    ( "Timestamp", "timestamp", _("Insert Ti_mestamp"), "<control><alt>M", _("Insert Timestamp"), inst.timestamp_insert),
+    ( "EmbFileInsert", "file_icon", _("Insert _File"), "<control><alt>E", _("Insert File"), inst.embfile_insert),
+    ( "BulletedList", "list_bulleted", _("Set/Unset _Bulleted List"), "<control><alt>1", _("Set/Unset the Current Paragraph/Selection as a Bulleted List"), inst.list_bulleted_handler),
+    ( "NumberedList", "list_numbered", _("Set/Unset _Numbered List"), "<control><alt>2", _("Set/Unset the Current Paragraph/Selection as a Numbered List"), inst.list_numbered_handler),
+    ( "ToDoList", "list_todo", _("Set/Unset _To-Do List"), "<control><alt>3", _("Set/Unset the Current Paragraph/Selection as a To-Do List"), inst.list_todo_handler),
     ( "HandleLink", "link_handle", _("Insert/Edit _Link"), "<control>L", _("Insert a Link/Edit the Underlying Link"), inst.apply_tag_link),
     ( "InsertTOC", "index", _("Insert T_OC"), None, _("Insert Table of Contents"), inst.toc_insert),
-    ( "HandleAnchor", "anchor_insert", _("Insert _Anchor"), None, _("Insert an Anchor"), inst.anchor_handle),
+    ( "HandleAnchor", "anchor_insert", _("Insert _Anchor"), "<control><alt>A", _("Insert an Anchor"), inst.anchor_handle),
     ( "EditAnchor", "anchor_edit", _("Edit _Anchor"), None, _("Edit the Underlying Anchor"), inst.anchor_edit),
-    ( "HandleImage", "image_insert", _("Insert I_mage"), None, _("Insert an Image"), inst.image_handle),
+    ( "EmbFileSave", "gtk-save-as", _("Save _As"), None, _("Save File As"), inst.embfile_save),
+    ( "HandleImage", "image_insert", _("Insert I_mage"), "<control><alt>I", _("Insert an Image"), inst.image_handle),
     ( "SaveImage", "image_save", _("_Save Image as PNG"), None, _("Save the Selected Image as a PNG file"), inst.image_save),
     ( "EditImage", "image_edit", _("_Edit Image"), None, _("Edit the Selected Image"), inst.image_edit),
     ( "CutImage", "edit-cut", _("C_ut Image"), None, _("Cut the Selected Image"), inst.image_cut),
     ( "CopyImage", "edit-copy", _("_Copy Image"), None, _("Copy the Selected Image"), inst.image_copy),
     ( "DeleteImage", "edit-delete", _("_Delete Image"), None, _("Delete the Selected Image"), inst.image_delete),
-    ( "CutTable", "edit-cut", _("C_ut Table"), None, _("Cut the Selected Table"), inst.tables_handler.table_cut),
-    ( "CopyTable", "edit-copy", _("_Copy Table"), None, _("Copy the Selected Table"), inst.tables_handler.table_copy),
-    ( "DeleteTable", "edit-delete", _("_Delete Table"), None, _("Delete the Selected Table"), inst.tables_handler.table_delete),
-    ( "HandleTable", "table_insert", _("Insert _Table"), None, _("Insert a Table"), inst.table_handle),
-    ( "HandleCodeBox", "codebox_insert", _("Insert _CodeBox"), None, _("Insert a CodeBox"), inst.codebox_handle),
+    ( "EditImageLink", "link_handle", _("Edit _Link"), None, _("Edit the Link Associated to the Image"), inst.image_link_edit),
+    ( "DismissImageLink", "gtk-clear", _("D_ismiss Link"), None, _("Dismiss the Link Associated to the Image"), inst.image_link_dismiss),
+    ( "HandleTable", "table_insert", _("Insert _Table"), "<control><alt>T", _("Insert a Table"), inst.table_handle),
+    ( "HandleCodeBox", "codebox_insert", _("Insert _CodeBox"), "<control><alt>C", _("Insert a CodeBox"), inst.codebox_handle),
     ( "DownCase", "case_lower", _("_Lower Case of Selection/Word"), "<control>W", _("Lower the Case of the Selection/the Underlying Word"), inst.text_selection_lower_case),
     ( "UpCase", "case_upper", _("_Upper Case of Selection/Word"), "<control><shift>W", _("Upper the Case of the Selection/the Underlying Word"), inst.text_selection_upper_case),
     ( "ToggleCase", "case_toggle", _("_Toggle Case of Selection/Word"), "<control>G", _("Toggle the Case of the Selection/the Underlying Word"), inst.text_selection_toggle_case),
@@ -565,8 +573,8 @@ def get_entries(inst):
     ( "FormatLatest", "format_text_latest", _("Format _Latest"), "F7", _("Memory of Latest Text Format Type"), inst.apply_tag_latest),
     ( "PastePlainText", "edit-paste", _("_Paste as Plain Text"), "<control><shift>V", _("Paste as Plain Text, Discard the Rich Text Formatting"), inst.paste_as_plain_text),
     ( "RemoveFormatting", "format_text_clear", _("_Remove Formatting"), "<control><shift>R", _("Remove the Formatting from the Selected Text"), inst.remove_text_formatting),
-    ( "ColorForeground", "color_foreground", _("Text _Color Foreground"), None, _("Change the Color of the Selected Text Foreground"), inst.apply_tag_foreground),
-    ( "ColorBackground", "color_background", _("Text C_olor Background"), None, _("Change the Color of the Selected Text Background"), inst.apply_tag_background),
+    ( "ColorForeground", "color_foreground", _("Text _Color Foreground"), "<shift><alt>F", _("Change the Color of the Selected Text Foreground"), inst.apply_tag_foreground),
+    ( "ColorBackground", "color_background", _("Text C_olor Background"), "<shift><alt>B", _("Change the Color of the Selected Text Background"), inst.apply_tag_background),
     ( "Bold", "format-text-bold", _("Toggle _Bold Property"), "<control>B", _("Toggle Bold Property of the Selected Text"), inst.apply_tag_bold),
     ( "Italic", "format-text-italic", _("Toggle _Italic Property"), "<control>I", _("Toggle Italic Property of the Selected Text"), inst.apply_tag_italic),
     ( "Underline", "format-text-underline", _("Toggle _Underline Property"), "<control>U", _("Toggle Underline Property of the Selected Text"), inst.apply_tag_underline),
@@ -577,27 +585,30 @@ def get_entries(inst):
     ( "Small", "format-text-small", _("Toggle _Small Property"), "<control>0", _("Toggle Small Property of the Selected Text"), inst.apply_tag_small),
     ( "Superscript", "format-text-superscript", _("Toggle Su_perscript Property"), None, _("Toggle Superscript Property of the Selected Text"), inst.apply_tag_superscript),
     ( "Subscript", "format-text-subscript", _("Toggle Su_bscript Property"), None, _("Toggle Subscript Property of the Selected Text"), inst.apply_tag_subscript),
-    ( "Monospace", "format-text-monospace", _("Toggle _Monospace Property"), None, _("Toggle Monospace Property of the Selected Text"), inst.apply_tag_monospace),
+    ( "Monospace", "format-text-monospace", _("Toggle _Monospace Property"), "<control>M", _("Toggle Monospace Property of the Selected Text"), inst.apply_tag_monospace),
     ( "JustifyLeft", "gtk-justify-left", _("Justify _Left"), None, _("Justify Left the Current Paragraph"), inst.apply_tag_justify_left),
     ( "JustifyCenter", "gtk-justify-center", _("Justify _Center"), None, _("Justify Center the Current Paragraph"), inst.apply_tag_justify_center),
     ( "JustifyRight", "gtk-justify-right", _("Justify _Right"), None, _("Justify Right the Current Paragraph"), inst.apply_tag_justify_right),
     ( "FindInNode", "find", _("_Find in Node"), "<control>F", _("Find into the Selected Node"), inst.find_in_selected_node),
-    ( "FindInNodes", "find", _("Find in Node_s"), "<control><shift>F", _("Find into all the Tree Nodes"), inst.find_in_all_nodes),
+    ( "FindInNodes", "find", _("Find in _All Nodes"), "<control><shift>F", _("Find into all the Tree Nodes"), inst.find_in_all_nodes),
+    ( "FindInSelNSub", "find", _("Find in _Selected Node and Subnodes"), "<control><alt>F", _("Find into the Selected Node and Subnodes"), inst.find_in_sel_node_and_subnodes),
     ( "FindNode", "find", _("Find a _Node"), "<control>T", _("Find a Node from its Name"), inst.find_a_node),
     ( "FindAgain", "find_again", _("Find _Again"), "F3", _("Iterate the Last Find Operation"), inst.find_again),
     ( "FindBack", "find_back", _("Find _Back"), "F4", _("Iterate the Last Find Operation in Opposite Direction"), inst.find_back),
     ( "ReplaceInNode", "find_replace", _("_Replace in Node"), "<control>H", _("Replace into the Selected Node"), inst.replace_in_selected_node),
-    ( "ReplaceInNodes", "find_replace", _("Replace in Node_s"), "<control><shift>H", _("Replace into all the Tree Nodes"), inst.replace_in_all_nodes),
+    ( "ReplaceInNodes", "find_replace", _("Replace in _All Nodes"), "<control><shift>H", _("Replace into all the Tree Nodes"), inst.replace_in_all_nodes),
+    ( "ReplaceInSelNSub", "find_replace", _("Replace in _Selected Node and Subnodes"), "<control><alt>H", _("Replace into the Selected Node and Subnodes"), inst.replace_in_sel_node_and_subnodes),
     ( "ReplaceInNodesNames", "find_replace", _("Replace in Nodes _Names"), "<control><shift>T", _("Replace in Nodes Names"), inst.replace_in_nodes_names),
     ( "ReplaceAgain", "find_replace", _("Replace _Again"), "F6", _("Iterate the Last Replace Operation"), inst.replace_again),
     ( "ShowHideNodeNameHeader", "node_name_header", _("Show/Hide Node Name _Header"), None, _("Toggle Show/Hide Node Name Header"), inst.toggle_show_hide_node_name_header),
+    ( "EnaDisSpellCheck", "gtk-spell-check", _("Enable/Disable _Spell Check"), "<control><alt>S", _("Toggle Enable/Disable Spell Check"), inst.toggle_ena_dis_spellcheck),
     ( "ShowHideTree", "cherries", _("Show/Hide _Tree"), "F9", _("Toggle Show/Hide Tree"), inst.toggle_show_hide_tree),
     ( "ShowHideMainWin", APP_NAME, _("Show/Hide _CherryTree"), None, _("Toggle Show/Hide CherryTree"), inst.toggle_show_hide_main_window),
     ( "ShowHideToolbar", "toolbar", _("Show/Hide Tool_bar"), None, _("Toggle Show/Hide Toolbar"), inst.toggle_show_hide_toolbar),
     ( "ShowAllMatchesDialog", "find", _("Show _All Matches Dialog"), "<control><shift>A", _("Show Search All Matches Dialog"), inst.find_allmatchesdialog_restore),
     ( "IncreaseToolbarIconsSize", "gtk-add", _("_Increase Toolbar Icons Size"), None, _("Increase the Size of the Toolbar Icons"), inst.toolbar_icons_size_increase),
     ( "DecreaseToolbarIconsSize", "gtk-remove", _("_Decrease Toolbar Icons Size"), None, _("Decrease the Size of the Toolbar Icons"), inst.toolbar_icons_size_decrease),
-    ( "ToggleTreeText", "gtk-jump-to", _("Toggle _Focus Tree/Text"), "<control>J", _("Toggle Focus Between Tree and Text"), inst.toggle_tree_text),
+    ( "ToggleTreeText", "gtk-jump-to", _("Toggle _Focus Tree/Text"), "<control>Tab", _("Toggle Focus Between Tree and Text"), inst.toggle_tree_text),
     ( "ToggleNodeExpColl", "gtk-zoom-in", _("Toggle Node _Expanded/Collapsed"), "<control><shift>J", _("Toggle Expanded/Collapsed Status of the Selected Node"), inst.toggle_tree_node_expanded_collapsed),
     ( "NodesExpAll", "gtk-zoom-in", _("E_xpand All Nodes"), "<control><shift>E", _("Expand All the Tree Nodes"), inst.nodes_expand_all),
     ( "NodesCollAll", "gtk-zoom-out", _("_Collapse All Nodes"), "<control><shift>L", _("Collapse All the Tree Nodes"), inst.nodes_collapse_all),
@@ -605,7 +616,10 @@ def get_entries(inst):
     ( "FromCherryTree", "from_cherrytree", _("From _CherryTree File"), None, _("Add Nodes of a CherryTree File to the Current Tree"), inst.nodes_add_from_cherrytree_file),
     ( "FromTxtFile", "from_txt", _("From _Plain Text File"), None, _("Add Node from a Plain Text File to the Current Tree"), inst.nodes_add_from_plain_text_file),
     ( "FromTxtFolder", "from_txt", _("From _Folder of Plain Text Files"), None, _("Add Nodes from a Folder of Plain Text Files to the Current Tree"), inst.nodes_add_from_plain_text_folder),
+    ( "FromHtmlFile", "from_html", _("From _HTML File"), None, _("Add Node from an HTML File to the Current Tree"), inst.nodes_add_from_html_file),
+    ( "FromHtmlFolder", "from_html", _("From _Folder of HTML Files"), None, _("Add Nodes from a Folder of HTML Files to the Current Tree"), inst.nodes_add_from_html_folder),
     ( "FromBasket", STR_STOCK_CT_IMP, _("From _Basket Folder"), None, _("Add Nodes of a Basket Folder to the Current Tree"), inst.nodes_add_from_basket_folder),
+    ( "FromEPIMHTML", STR_STOCK_CT_IMP, _("From _EssentialPIM HTML File"), None, _("Add Node from an EssentialPIM HTML File to the Current Tree"), inst.nodes_add_from_epim_html_file),
     ( "FromGnote", STR_STOCK_CT_IMP, _("From _Gnote Folder"), None, _("Add Nodes of a Gnote Folder to the Current Tree"), inst.nodes_add_from_gnote_folder),
     ( "FromKeepNote", STR_STOCK_CT_IMP, _("From _KeepNote Folder"), None, _("Add Nodes of a KeepNote Folder to the Current Tree"), inst.nodes_add_from_keepnote_folder),
     ( "FromKeyNote", STR_STOCK_CT_IMP, _("From K_eyNote File"), None, _("Add Nodes of a KeyNote File to the Current Tree"), inst.nodes_add_from_keynote_file),
@@ -620,23 +634,38 @@ def get_entries(inst):
     ( "Help", "help-contents", _("Online _Manual"), "F1", _("Application's Online Manual"), inst.on_help_menu_item_activated),
     ( "CheckNewer", "gtk-network", _("_Check Newer Version"), None, _("Check for a Newer Version"), inst.check_for_newer_version),
     ( "About", "gtk-about", _("_About"), None, _("About CherryTree"), inst.dialog_about),
-    ( "TableEditProp", "table_edit", _("_Edit Table Properties"), None, _("Edit the Table Properties"), inst.tables_handler.table_edit_properties),
-    ( "TableRowAdd", "gtk-add", _("_Add Row"), None, _("Add a Table Row"), inst.tables_handler.table_row_add),
-    ( "TableRowCut", "edit-cut", _("Cu_t Row"), None, _("Cut a Table Row"), inst.tables_handler.table_row_cut),
-    ( "TableRowCopy", "edit-copy", _("_Copy Row"), None, _("Copy a Table Row"), inst.tables_handler.table_row_copy),
-    ( "TableRowPaste", "edit-paste", _("_Paste Row"), None, _("Paste a Table Row"), inst.tables_handler.table_row_paste),
-    ( "TableRowDelete", "edit-delete", _("De_lete Row"), None, _("Delete the Selected Table Row"), inst.tables_handler.table_row_delete),
-    ( "TableRowUp", "gtk-go-up", _("Move Row _Up"), None, _("Move the Selected Row Up"), inst.tables_handler.table_row_up),
-    ( "TableRowDown", "gtk-go-down", _("Move Row _Down"), None, _("Move the Selected Row Down"), inst.tables_handler.table_row_down),
-    ( "TableSortRowsDesc", "gtk-sort-descending", _("Sort Rows De_scending"), None, _("Sort all the Rows Descending"), inst.tables_handler.table_rows_sort_descending),
-    ( "TableSortRowsAsc", "gtk-sort-ascending", _("Sort Rows As_cending"), None, _("Sort all the Rows Ascending"), inst.tables_handler.table_rows_sort_ascending),
-    ( "TableExport", "table_save", _("_Table Export"), None, _("Export Table as CSV File"), inst.tables_handler.table_export),
     ( "BookmarkThisNode", "gtk-add", _("_Bookmark This Node"), None, _("Add the Current Node to the Bookmarks List"), inst.bookmark_curr_node),
     ( "BookmarksHandle", "gtk-edit", _("_Handle Bookmarks"), None, _("Handle the Bookmarks List"), inst.bookmarks_handle),
     ( "TreeAddNode", "tree-node-add", _("Add _Node"), "<control>N", _("Add a Node having the same Father of the Selected Node"), inst.node_add),
     ( "TreeAddSubNode", "tree-subnode-add", _("Add _SubNode"), "<control><shift>N", _("Add a Child Node to the Selected Node"), inst.node_child_add),
     ]
 
+def get_popup_menu_table(inst):
+    """Returns the Menu Entries Given the Class Instance"""
+    # stock id, label, accelerator, tooltip, callback |
+    # "separator", None, None, None, None |
+    # "submenu-start", label, stock id, None, None |
+    # "submenu-end", None, None, None, None
+    return [
+    ("edit-cut", _("C_ut Table"), None, _("Cut the Selected Table"), inst.tables_handler.table_cut),
+    ("edit-copy", _("_Copy Table"), None, _("Copy the Selected Table"), inst.tables_handler.table_copy),
+    ("edit-delete", _("_Delete Table"), None, _("Delete the Selected Table"), inst.tables_handler.table_delete),
+    ("separator", None, None, None, None),
+    ("gtk-add", _("_Add Row"), "<control>comma", _("Add a Table Row"), inst.tables_handler.table_row_add),
+    ("edit-cut", _("Cu_t Row"), None, _("Cut a Table Row"), inst.tables_handler.table_row_cut),
+    ("edit-copy", _("_Copy Row"), None, _("Copy a Table Row"), inst.tables_handler.table_row_copy),
+    ("edit-paste", _("_Paste Row"), None, _("Paste a Table Row"), inst.tables_handler.table_row_paste),
+    ("edit-delete", _("De_lete Row"), "<control><alt>comma", _("Delete the Selected Table Row"), inst.tables_handler.table_row_delete),
+    ("separator", None, None, None, None),
+    ("gtk-go-up", _("Move Row _Up"), "<control><alt>period", _("Move the Selected Row Up"), inst.tables_handler.table_row_up),
+    ("gtk-go-down", _("Move Row _Down"), "<control>period", _("Move the Selected Row Down"), inst.tables_handler.table_row_down),
+    ("gtk-sort-descending", _("Sort Rows De_scending"), None, _("Sort all the Rows Descending"), inst.tables_handler.table_rows_sort_descending),
+    ("gtk-sort-ascending", _("Sort Rows As_cending"), None, _("Sort all the Rows Ascending"), inst.tables_handler.table_rows_sort_ascending),
+    ("separator", None, None, None, None),
+    ("table_edit", _("_Edit Table Properties"), None, _("Edit the Table Properties"), inst.tables_handler.table_edit_properties),
+    ("table_save", _("_Table Export"), None, _("Export Table as CSV File"), inst.tables_handler.table_export),
+    ]
+
 def get_popup_menu_tree(inst):
     """Returns the Menu Entries Given the Class Instance"""
     # stock id, label, accelerator, tooltip, callback |
@@ -672,8 +701,11 @@ def get_popup_menu_tree(inst):
     ("from_cherrytree", _("From _CherryTree File"), None, _("Add Nodes of a CherryTree File to the Current Tree"), inst.nodes_add_from_cherrytree_file),
     ("from_txt", _("From _Plain Text File"), None, _("Add Node from a Plain Text File to the Current Tree"), inst.nodes_add_from_plain_text_file),
     ("from_txt", _("From _Folder of Plain Text Files"), None, _("Add Nodes from a Folder of Plain Text Files to the Current Tree"), inst.nodes_add_from_plain_text_folder),
+    ("from_html", _("From _HTML File"), None, _("Add Node from an HTML File to the Current Tree"), inst.nodes_add_from_html_file),
+    ("from_html", _("From _Folder of HTML Files"), None, _("Add Nodes from a Folder of HTML Files to the Current Tree"), inst.nodes_add_from_html_folder),
     (STR_STOCK_CT_IMP, _("From _Basket Folder"), None, _("Add Nodes of a Basket Folder to the Current Tree"), inst.nodes_add_from_basket_folder),
     (STR_STOCK_CT_IMP, _("From _Gnote Folder"), None, _("Add Nodes of a Gnote Folder to the Current Tree"), inst.nodes_add_from_gnote_folder),
+    (STR_STOCK_CT_IMP, _("From _EssentialPIM HTML File"), None, _("Add Node from an EssentialPIM HTML File to the Current Tree"), inst.nodes_add_from_epim_html_file),
     (STR_STOCK_CT_IMP, _("From _KeepNote Folder"), None, _("Add Nodes of a KeepNote Folder to the Current Tree"), inst.nodes_add_from_keepnote_folder),
     (STR_STOCK_CT_IMP, _("From K_eyNote File"), None, _("Add Nodes of a KeyNote File to the Current Tree"), inst.nodes_add_from_keynote_file),
     (STR_STOCK_CT_IMP, _("From K_nowit File"), None, _("Add Nodes of a Knowit File to the Current Tree"), inst.nodes_add_from_knowit_file),
@@ -727,7 +759,7 @@ def get_popup_menu_entries_text(inst):
     ("format-text-small", _("Toggle _Small Property"), "<control>0", _("Toggle Small Property of the Selected Text"), inst.apply_tag_small),
     ("format-text-superscript", _("Toggle Su_perscript Property"), None, _("Toggle Superscript Property of the Selected Text"), inst.apply_tag_superscript),
     ("format-text-subscript", _("Toggle Su_bscript Property"), None, _("Toggle Subscript Property of the Selected Text"), inst.apply_tag_subscript),
-    ("format-text-monospace", _("Toggle _Monospace Property"), None, _("Toggle Monospace Property of the Selected Text"), inst.apply_tag_monospace),
+    ("format-text-monospace", _("Toggle _Monospace Property"), "<control>M", _("Toggle Monospace Property of the Selected Text"), inst.apply_tag_monospace),
     ("submenu-end", None, None, None, None),
     ("submenu-start", _("_Justify") , "gtk-justify-center", None, None),
     ("gtk-justify-left", _("Justify _Left"), None, _("Justify Left the Current Paragraph"), inst.apply_tag_justify_left),
@@ -741,13 +773,14 @@ def get_popup_menu_entries_text(inst):
     ("submenu-end", None, None, None, None),
     ("separator", None, None, None, None),
     ("submenu-start", _("_Insert") , "insert", None, None),
-    ("image_insert", _("Insert I_mage"), None, _("Insert an Image"), inst.image_handle),
-    ("table_insert", _("Insert _Table"), None, _("Insert a Table"), inst.table_handle),
-    ("codebox_insert", _("Insert _CodeBox"), None, _("Insert a CodeBox"), inst.codebox_handle),
+    ("image_insert", _("Insert I_mage"), "<control><alt>I", _("Insert an Image"), inst.image_handle),
+    ("table_insert", _("Insert _Table"), "<control><alt>T", _("Insert a Table"), inst.table_handle),
+    ("codebox_insert", _("Insert _CodeBox"), "<control><alt>C", _("Insert a CodeBox"), inst.codebox_handle),
+    ("file_icon", _("Insert _File"), "<control><alt>E", _("Insert File"), inst.embfile_insert),
     ("link_handle", _("Insert/Edit _Link"), "<control>L", _("Insert a Link/Edit the Underlying Link"), inst.apply_tag_link),
-    ("anchor_insert", _("Insert _Anchor"), None, _("Insert an Anchor"), inst.anchor_handle),
+    ("anchor_insert", _("Insert _Anchor"), "<control><alt>A", _("Insert an Anchor"), inst.anchor_handle),
     ("index", _("Insert T_OC"), None, _("Insert Table of Contents"), inst.toc_insert),
-    ("timestamp", _("Insert Ti_mestamp"), "<control>M", _("Insert Timestamp"), inst.timestamp_insert),
+    ("timestamp", _("Insert Ti_mestamp"), "<control><alt>M", _("Insert Timestamp"), inst.timestamp_insert),
     ("horizontal_rule", _("Insert _Horizontal Rule"), "<control>R", _("Insert Horizontal Rule"), inst.horizontal_rule_insert),
     ("submenu-end", None, None, None, None),
     ("submenu-start", _("C_hange Case") , "case_toggle", None, None),
@@ -787,7 +820,7 @@ def get_popup_menu_entries_code(inst):
     # "submenu-end", None, None, None, None
     return [
     ("separator", None, None, None, None),
-    ("timestamp", _("Insert Ti_mestamp"), "<control>M", _("Insert Timestamp"), inst.timestamp_insert),
+    ("timestamp", _("Insert Ti_mestamp"), "<control><alt>M", _("Insert Timestamp"), inst.timestamp_insert),
     ("gtk-clear", _("Stri_p Trailing Spaces"), None, _("Strip Trailing Spaces"), inst.strip_trailing_spaces),
     ("submenu-start", _("C_hange Case") , "case_toggle", None, None),
     ("case_lower", _("_Lower Case of Selection/Word"), "<control>W", _("Lower the Case of the Selection/the Underlying Word"), inst.text_selection_lower_case),
@@ -834,6 +867,17 @@ def get_popup_menu_entries_link(inst):
     ("edit-delete", _("_Delete Link"), None, _("Delete the Selected Link"), inst.link_delete),
     ]
 
+def get_popup_menu_entries_table_cell(inst):
+    """Returns the Menu Entries Given the Class Instance"""
+    # stock id, label, accelerator, tooltip, callback |
+    # "separator", None, None, None, None |
+    # "submenu-start", label, stock id, None, None |
+    # "submenu-end", None, None, None, None
+    return [
+    ("separator", None, None, None, None),
+    ("insert", _("Insert _NewLine"), "<control>period", _("Insert NewLine Char"), inst.curr_table_cell_insert_newline),
+    ]
+
 def get_popup_menu_entries_codebox(inst):
     """Returns the Menu Entries Given the Class Instance"""
     # stock id, label, accelerator, tooltip, callback |
@@ -843,10 +887,13 @@ def get_popup_menu_entries_codebox(inst):
     return [
     ("separator", None, None, None, None),
     ("codebox_edit", _("Change CodeBox _Properties"), None, _("Edit the Properties of the CodeBox"), inst.codebox_change_properties),
+    ("from_txt", _("CodeBox _Load From Text File"), None, _("Load the CodeBox Content From a Text File"), inst.codebox_load_from_file),
+    ("to_txt", _("CodeBox _Save To Text File"), None, _("Save the CodeBox Content To a Text File"), inst.codebox_save_to_file),
     ("separator", None, None, None, None),
     ("edit-cut", _("C_ut CodeBox"), None, _("Cut the Selected CodeBox"), inst.codebox_cut),
     ("edit-copy", _("_Copy CodeBox"), None, _("Copy the Selected CodeBox"), inst.codebox_copy),
     ("edit-delete", _("_Delete CodeBox"), None, _("Delete the Selected CodeBox"), inst.codebox_delete),
+    ("edit-delete", _("Delete CodeBox _Keep Content"), None, _("Delete the Selected CodeBox But Keep Its Content"), inst.codebox_delete_keeping_text),
     ("separator", None, None, None, None),
     ("gtk-go-forward", _("Increase CodeBox Width"), "<control>period", _("Increase the Width of the CodeBox"), inst.codebox_increase_width),
     ("gtk-go-back", _("Decrease CodeBox Width"), "<control><alt>period", _("Decrease the Width of the CodeBox"), inst.codebox_decrease_width),
