@@ -72,13 +72,24 @@ class CTDBHandler:
         offset = image_element[0]
         pixbuf = image_element[1]
         justification = image_element[2]
-        if not "anchor" in dir(pixbuf):
-            anchor = ""
-            png_blob = machines.get_blob_buffer_from_pixbuf(pixbuf)
-        else:
+        pixbuf_attrs = dir(pixbuf)
+        if "anchor" in pixbuf_attrs:
+            filename = ""
+            link = ""
             anchor = (pixbuf.anchor).decode(cons.STR_UTF8)
             png_blob = None
-        return (node_id, offset, justification, anchor, png_blob)
+        elif "filename" in pixbuf_attrs:
+            filename = (pixbuf.filename).decode(cons.STR_UTF8)
+            link = ""
+            anchor = ""
+            png_blob = buffer(pixbuf.embfile)
+        else:
+            filename = ""
+            try: link = (pixbuf.link).decode(cons.STR_UTF8)
+            except: link = ""
+            anchor = ""
+            png_blob = machines.get_blob_buffer_from_pixbuf(pixbuf)
+        return (node_id, offset, justification, anchor, png_blob, filename, link)
     
     def get_table_db_tuple(self, table_element, node_id):
         """From table element to db tuple"""
@@ -147,14 +158,14 @@ class CTDBHandler:
     
     def pending_edit_db_bookmarks(self):
         """Pending Bookmarks Update"""
-        if self.dad.filetype not in ["b", "x"]: return
+        if self.dad.filetype not in ["b", "x", ""]: return
         if not self.dad.user_active: return
         print "pending_edit_db_bookmarks"
         self.bookmarks_to_write = True
     
     def pending_edit_db_node_prop(self, node_id):
         """Pending Node Needs 'prop' Update"""
-        if self.dad.filetype not in ["b", "x"]: return
+        if self.dad.filetype not in ["b", "x", ""]: return
         if not self.dad.user_active: return
         print "pending_edit_db_node_prop", node_id
         if node_id in self.nodes_to_write_dict:
@@ -165,7 +176,7 @@ class CTDBHandler:
     
     def pending_edit_db_node_buff(self, node_id, force_user_active=False):
         """Pending Node Needs 'buff' Update"""
-        if self.dad.filetype not in ["b", "x"]: return
+        if self.dad.filetype not in ["b", "x", ""]: return
         if not self.dad.user_active and not force_user_active: return
         print "pending_edit_db_node_buff", node_id
         if node_id in self.nodes_to_write_dict:
@@ -176,7 +187,7 @@ class CTDBHandler:
     
     def pending_edit_db_node_hier(self, node_id):
         """Pending Node Needs 'hier' Update"""
-        if self.dad.filetype not in ["b", "x"]: return
+        if self.dad.filetype not in ["b", "x", ""]: return
         if not self.dad.user_active: return
         print "pending_edit_db_node_hier", node_id
         if node_id in self.nodes_to_write_dict:
@@ -187,14 +198,14 @@ class CTDBHandler:
     
     def pending_new_db_node(self, node_id):
         """Pending Add a Node to DB"""
-        if self.dad.filetype not in ["b", "x"]: return
+        if self.dad.filetype not in ["b", "x", ""]: return
         print "pending_new_db_node", node_id
         write_dict = {'upd': False, 'prop': True, 'buff': True, 'hier': True, 'child': False}
         self.nodes_to_write_dict[node_id] = write_dict
     
     def pending_rm_db_node(self, node_id):
         """Pending RM a Node from DB"""
-        if self.dad.filetype not in ["b", "x"]: return
+        if self.dad.filetype not in ["b", "x", ""]: return
         if not self.dad.user_active: return
         print "pending_rm_db_node", node_id
         if node_id in self.nodes_to_write_dict:
@@ -208,9 +219,9 @@ class CTDBHandler:
                 while tree_iter_children:
                     self.pending_rm_just_added_node_children(tree_iter_children)
                     tree_iter_children = self.dad.treestore.iter_next(tree_iter_children)
-                # no need to rm the node, we just do not add it
-                return
+                # no need to rm the node, we just do not add it... but write to set to not use this id again
         self.nodes_to_rm_set.add(node_id)
+        #print self.nodes_to_rm_set
     
     def pending_rm_just_added_node_children(self, tree_iter):
         """Handle situation of nodes children just added and immediately removed"""
@@ -243,7 +254,7 @@ class CTDBHandler:
         syntax = self.dad.treestore[tree_iter][4]
         tags = self.dad.treestore[tree_iter][6].decode(cons.STR_UTF8)
         is_ro = 1 if self.dad.treestore[tree_iter][7] else 0
-        is_richtxt = 1 if syntax == cons.CUSTOM_COLORS_ID else 0
+        is_richtxt = 1 if syntax == cons.RICH_TEXT_ID else 0
         if write_dict['buff']:
             if not self.dad.treestore[tree_iter][2]:
                 # we are using db storage and the buffer was not created yet
@@ -304,7 +315,10 @@ class CTDBHandler:
                 else: has_table = 0
                 if images_tuples:
                     has_image = 1
-                    db.executemany('INSERT INTO image VALUES(?,?,?,?,?)', images_tuples)
+                    if len(db.execute('PRAGMA table_info(image)').fetchall()) == 5:
+                        db.execute('ALTER TABLE image ADD COLUMN filename TEXT')
+                        db.execute('ALTER TABLE image ADD COLUMN link TEXT')
+                    db.executemany('INSERT INTO image VALUES(?,?,?,?,?,?,?)', images_tuples)
                 else: has_image = 0
                 # retrieve xml text
                 txt = (self.dom.toxml()).decode(cons.STR_UTF8)
@@ -419,15 +433,20 @@ class CTDBHandler:
         """Add Image to Text Buffer"""
         iter_insert = text_buffer.get_iter_at_offset(image_row['offset'])
         if image_row['anchor']:
-            pixbuf = gtk.gdk.pixbuf_new_from_file(cons.ANCHOR_CHAR)
-            pixbuf = pixbuf.scale_simple(self.dad.anchor_size, self.dad.anchor_size, gtk.gdk.INTERP_BILINEAR)
+            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(cons.ANCHOR_CHAR, self.dad.anchor_size, self.dad.anchor_size)
             pixbuf.anchor = image_row['anchor']
-        else: pixbuf = machines.get_pixbuf_from_png_blob_buffer(image_row['png'])
+        elif 'filename' in image_row.keys() and image_row['filename']:
+            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(cons.FILE_CHAR, self.dad.embfile_size, self.dad.embfile_size)
+            pixbuf.filename = image_row['filename']
+            pixbuf.embfile = image_row['png']
+        else:
+            pixbuf = machines.get_pixbuf_from_png_blob_buffer(image_row['png'])
+            pixbuf.link = image_row['link'] if 'link' in image_row.keys() else ""
         if pixbuf:
             self.dad.image_insert(iter_insert,
-                                  pixbuf,
-                                  image_justification=image_row['justification'],
-                                  text_buffer=text_buffer)
+                pixbuf,
+                image_justification=image_row['justification'],
+                text_buffer=text_buffer)
     
     def read_db_node_content(self, tree_iter, db, original_id=None):
         """Read a node content from DB"""
@@ -442,7 +461,7 @@ class CTDBHandler:
         self.dad.treestore[tree_iter][2] = self.dad.buffer_create(syntax_highlighting)
         curr_buffer = self.dad.treestore[tree_iter][2]
         node_row = db.execute('SELECT txt, has_codebox, has_table, has_image FROM node WHERE node_id=?', (node_id,)).fetchone()
-        if syntax_highlighting != cons.CUSTOM_COLORS_ID:
+        if syntax_highlighting != cons.RICH_TEXT_ID:
             curr_buffer.begin_not_undoable_action()
             curr_buffer.set_text(node_row['txt'])
             curr_buffer.end_not_undoable_action()
@@ -519,10 +538,11 @@ class CTDBHandler:
         if node_tags: self.dad.tags_add_from_node(node_tags)
         readonly = node_row['is_ro']
         syntax_highlighting = node_row['syntax']
-        if syntax_highlighting != cons.CUSTOM_COLORS_ID and syntax_highlighting not in self.dad.available_languages:
+        if syntax_highlighting not in [cons.RICH_TEXT_ID, cons.PLAIN_TEXT_ID]\
+        and syntax_highlighting not in self.dad.available_languages:
             syntax_highlighting = syntax_highlighting.lower().replace("C++", "cpp")
             if syntax_highlighting not in self.dad.available_languages:
-                syntax_highlighting = cons.CUSTOM_COLORS_ID
+                syntax_highlighting = cons.RICH_TEXT_ID
         node_level = self.dad.treestore.iter_depth(tree_father)+1 if tree_father else 0
         cherry = self.dad.get_node_icon(node_level, syntax_highlighting)
         #print "added node with unique_id", unique_id
@@ -555,9 +575,6 @@ class CTDBHandler:
     
     def read_db_full(self, db, discard_ids, tree_father=None):
         """Read the whole DB"""
-        if not discard_ids:
-            self.dad.xml_handler.reset_nodes_names()
-            self.dad.bookmarks = []
         # tree nodes
         node_sequence = 0
         children_rows = self.get_children_rows_from_father_id(db, 0)
