@@ -95,10 +95,10 @@ class ExportPrint:
     def get_pdf_filepath(self, proposed_name):
         """Dialog to select dest PDF"""
         ret_filepath = support.dialog_file_save_as(proposed_name + ".pdf",
-                                                   filter_pattern="*.pdf",
-                                                   filter_name=_("PDF File"),
-                                                   curr_folder=self.dad.pick_dir,
-                                                   parent=self.dad.window)
+            filter_pattern="*.pdf",
+            filter_name=_("PDF File"),
+            curr_folder=self.dad.pick_dir,
+            parent=self.dad.window)
         if ret_filepath:
             if not ret_filepath.endswith(".pdf"): ret_filepath += ".pdf"
             self.dad.pick_dir = os.path.dirname(ret_filepath)
@@ -121,7 +121,7 @@ class ExportPrint:
     def nodes_all_export_print_iter(self, tree_iter, include_node_name, new_node_in_new_page):
         """Export Print All Nodes - Iter"""
         self.dad.get_textbuffer_from_tree_iter(tree_iter)
-        if self.dad.treestore[tree_iter][4] == cons.CUSTOM_COLORS_ID:
+        if self.dad.treestore[tree_iter][4] == cons.RICH_TEXT_ID:
             pango_text, pixbuf_table_codebox_vector = self.pango_handler.pango_get_from_treestore_node(tree_iter)
             self.text_font = self.dad.text_font # text font for all (also eventual code nodes)
         else:
@@ -151,24 +151,24 @@ class ExportPrint:
 
     def node_export_print(self, tree_iter, include_node_name, sel_range=None):
         """Export Print the Selected Node"""
-        if self.dad.treestore[tree_iter][4] == cons.CUSTOM_COLORS_ID:
+        if self.dad.treestore[tree_iter][4] == cons.RICH_TEXT_ID:
             self.pango_text, self.pixbuf_table_codebox_vector = self.pango_handler.pango_get_from_treestore_node(tree_iter, sel_range)
             self.text_font = self.dad.text_font
         else:
             self.pango_text = [self.pango_handler.pango_get_from_code_buffer(self.dad.treestore[tree_iter][2], sel_range)]
             self.pixbuf_table_codebox_vector = []
-            self.text_font = self.dad.code_font
+            self.text_font = self.dad.code_font if self.dad.treestore[tree_iter][4] != cons.PLAIN_TEXT_ID else self.dad.text_font
         if include_node_name: self.pango_text_add_node_name(tree_iter, self.pango_text)
         self.run_print()
 
     def run_print(self):
         """Finally Run the Print"""
         self.dad.print_handler.print_text(self.dad.window,
-                                          self.pango_text,
-                                          self.text_font,
-                                          self.dad.code_font,
-                                          self.pixbuf_table_codebox_vector,
-                                          self.dad.get_text_window_width())
+            self.pango_text,
+            self.text_font,
+            self.dad.code_font,
+            self.pixbuf_table_codebox_vector,
+            self.dad.get_text_window_width())
 
 
 class Export2Txt:
@@ -258,9 +258,7 @@ class Export2Txt:
 
     def plain_get_from_treestore_node(self, curr_buffer, sel_range=None):
         """Given a treestore iter returns the HTML rich text"""
-        pixbuf_table_codebox_vector = self.dad.state_machine.get_embedded_pixbufs_tables_codeboxes(curr_buffer,
-                                                                                                   for_print=2,
-                                                                                                   sel_range=sel_range)
+        pixbuf_table_codebox_vector = self.dad.state_machine.get_embedded_pixbufs_tables_codeboxes(curr_buffer, sel_range=sel_range)
         # pixbuf_table_codebox_vector is [ [ "pixbuf"/"table"/"codebox", [offset, pixbuf, alignment] ],... ]
         self.curr_plain_slots = []
         start_offset = 0 if not sel_range else sel_range[0]
@@ -287,9 +285,11 @@ class Export2Txt:
                 elif curr_object[0] == "codebox": plain_text += self.get_codebox_plain(curr_object[1])
         if tree_iter_for_node_name:
             plain_text = self.plain_text_get_node_name(tree_iter_for_node_name) + plain_text
-        file_descriptor = open(filepath, 'a')
-        file_descriptor.write(plain_text + 2*cons.CHAR_NEWLINE)
-        file_descriptor.close()
+        if filepath:
+            file_descriptor = open(filepath, 'a')
+            file_descriptor.write(plain_text + 2*cons.CHAR_NEWLINE)
+            file_descriptor.close()
+        return plain_text
     
     def plain_text_get_node_name(self, tree_iter):
         """Get Node Name in Plain Text"""
@@ -349,8 +349,8 @@ class Export2Pango:
         """Given a treestore iter returns the Pango rich text"""
         curr_buffer = self.dad.treestore[node_iter][2]
         pixbuf_table_codebox_vector = self.dad.state_machine.get_embedded_pixbufs_tables_codeboxes(curr_buffer,
-                                                                                                   for_print=1,
-                                                                                                   sel_range=sel_range)
+            for_print=1,
+            sel_range=sel_range)
         # pixbuf_table_codebox_vector is [ [ "pixbuf"/"table"/"codebox", [offset, pixbuf, alignment] ],... ]
         self.curr_pango_slots = []
         start_offset = 0 if not sel_range else sel_range[0]
@@ -460,6 +460,7 @@ class Export2Html:
         self.tree_links_nums = ["1"]
         for image_stock_id in cons.NODES_STOCKS:
             shutil.copy(cons.GLADE_PATH + image_stock_id + ".png", self.images_dir)
+        shutil.copy(cons.GLADE_PATH + "home.png", self.images_dir)
         pango_font = pango.FontDescription(self.dad.tree_font)
         self.tree_links_text = '<table style="text-align:left; font-family: %s; font-size:%spt">' % (pango_font.get_family(), pango_font.get_size()/pango.SCALE)
         if not top_tree_iter: tree_iter = self.dad.treestore.get_iter_first()
@@ -490,6 +491,16 @@ class Export2Html:
             self.nodes_all_export_to_html_iter(child_tree_iter)
             child_tree_iter = self.dad.treestore.iter_next(child_tree_iter)
 
+    def clean_text_to_utf8(self, in_text):
+        """Clean from utf8 decode errors"""
+        clean_text = in_text
+        while clean_text:
+            try:
+                clean_text = unicode(clean_text, cons.STR_UTF8, cons.STR_IGNORE)
+                break
+            except: clean_text = clean_text[1:]
+        return clean_text
+
     def tree_links_text_iter(self, tree_iter):
         """Creating the Tree Links Text - iter"""
         href = self.get_html_filename(tree_iter)
@@ -497,7 +508,8 @@ class Export2Html:
         self.tree_links_text += "&nbsp;&nbsp;&nbsp;" * len(self.tree_links_nums)
         icon_rel_path = os.path.join("images", self.dad.treestore[tree_iter][0] + ".png")
         self.tree_links_text += '<img src="%s" alt="%s" height="22" width="22"/>' % (icon_rel_path, icon_rel_path)
-        self.tree_links_text += '<a href="' + href + '">' + ".".join(self.tree_links_nums) + " " + self.dad.treestore[tree_iter][1] + "</a></td></tr>"
+        node_name = self.clean_text_to_utf8(self.dad.treestore[tree_iter][1])
+        self.tree_links_text += '<a href="' + href + '">' + ".".join(self.tree_links_nums) + " " + node_name + "</a></td></tr>"
         child_tree_iter = self.dad.treestore.iter_children(tree_iter)
         self.tree_links_nums.append("1")
         while child_tree_iter:
@@ -522,15 +534,15 @@ class Export2Html:
             sel_range = [iter_start.get_offset(), iter_end.get_offset()]
         else: sel_range = None
         html_text = cons.HTML_HEADER % self.dad.treestore[tree_iter][1]
-        if self.tree_links_text:
+        if self.tree_links_text and self.dad.last_index_in_page:
             html_text += r'<table width="100%"><tr>'
             td_tree = r'<td valign="top" align=cons.TAG_PROP_LEFT width=20%>'
             td_page = r'<td valign="top" align=cons.TAG_PROP_LEFT width=80%>'
             html_text += td_tree + self.tree_links_text + td_page
-        pango_font = pango.FontDescription(self.dad.text_font if self.dad.treestore[tree_iter][4] == cons.CUSTOM_COLORS_ID else self.dad.code_font)
+        pango_font = pango.FontDescription(self.dad.text_font if self.dad.treestore[tree_iter][4] in [cons.RICH_TEXT_ID, cons.PLAIN_TEXT_ID] else self.dad.code_font)
         html_text += '<span style="font-family: %s; font-size:%spt">' % (pango_font.get_family(), pango_font.get_size()/pango.SCALE)
         self.dad.get_textbuffer_from_tree_iter(tree_iter)
-        if self.dad.treestore[tree_iter][4] == cons.CUSTOM_COLORS_ID:
+        if self.dad.treestore[tree_iter][4] == cons.RICH_TEXT_ID:
             text_n_objects = self.html_get_from_treestore_node(tree_iter, sel_range)
             self.images_count = 0
             for i, html_slot in enumerate(text_n_objects[0]):
@@ -541,8 +553,10 @@ class Export2Html:
                     elif curr_object[0] == "table": html_text += self.get_table_html(curr_object[1])
                     elif curr_object[0] == "codebox": html_text += self.get_codebox_html(curr_object[1])
         else: html_text += self.html_get_from_code_buffer(self.dad.treestore[tree_iter][2], sel_range)
+        if self.tree_links_text and not self.dad.last_index_in_page:
+            html_text += '<p align="center">' + '<img src="%s" height="22" width="22">' % os.path.join("images", "home.png") + 2*cons.CHAR_SPACE + '<a href="index.html">' + _("Index") + '</a></p>'
         html_text += "</span>"
-        if self.tree_links_text:
+        if self.tree_links_text and self.dad.last_index_in_page:
             html_text += '</td></tr></table>'
         html_text += cons.HTML_FOOTER
         node_html_filepath = os.path.join(self.new_path, self.get_html_filename(tree_iter))
@@ -554,7 +568,7 @@ class Export2Html:
     def selection_export_to_html(self, text_buffer, start_iter, end_iter, syntax_highlighting):
         """Returns the HTML given the text buffer and iter bounds"""
         html_text = cons.HTML_HEADER % ""
-        if syntax_highlighting == cons.CUSTOM_COLORS_ID:
+        if syntax_highlighting == cons.RICH_TEXT_ID:
             text_n_objects = self.html_get_from_rich_text_selection(text_buffer, start_iter, end_iter)
             self.images_count = 0
             for i, html_slot in enumerate(text_n_objects[0]):
@@ -638,10 +652,10 @@ class Export2Html:
 
     def get_html_filename(self, tree_iter):
         """Get the HTML page filename given the tree iter"""
-        file_name = self.dad.treestore[tree_iter][1].strip()
+        file_name = self.clean_text_to_utf8(self.dad.treestore[tree_iter][1]).strip()
         father_iter = self.dad.treestore.iter_parent(tree_iter)
         while father_iter:
-            file_name = self.dad.treestore[father_iter][1].strip() + "--" + file_name
+            file_name = self.clean_text_to_utf8(self.dad.treestore[father_iter][1]).strip() + "--" + file_name
             father_iter = self.dad.treestore.iter_parent(father_iter)
         file_name = support.clean_from_chars_not_for_filename(file_name) + ".html"
         if len(file_name) > cons.MAX_FILE_NAME_LEN:
