@@ -99,9 +99,6 @@ class XMLHandler:
         """Parse an XML Cherry Tree Document file and build the Tree"""
         dom = xml.dom.minidom.parseString(ctd)
         cherrytree = dom.firstChild
-        if not discard_ids:
-            self.reset_nodes_names()
-            self.dad.bookmarks = []
         if cherrytree.nodeName != cons.APP_NAME: return False
         else:
             dom_iter = cherrytree.firstChild
@@ -126,14 +123,15 @@ class XMLHandler:
         if dom_iter.hasAttribute('readonly'): readonly = (dom_iter.attributes['readonly'].value == "True")
         else: readonly = False
         syntax_highlighting = dom_iter.attributes['prog_lang'].value
-        if syntax_highlighting != cons.CUSTOM_COLORS_ID and syntax_highlighting not in self.dad.available_languages:
+        if syntax_highlighting not in [cons.RICH_TEXT_ID, cons.PLAIN_TEXT_ID]\
+        and syntax_highlighting not in self.dad.available_languages:
             syntax_highlighting = syntax_highlighting.lower().replace("C++", "cpp")
             if syntax_highlighting not in self.dad.available_languages:
-                syntax_highlighting = cons.CUSTOM_COLORS_ID
+                syntax_highlighting = cons.RICH_TEXT_ID
         node_depth = 0 if not tree_father else self.dad.treestore.iter_depth(tree_father)+1
         cherry = self.dad.get_node_icon(node_depth, syntax_highlighting)
         curr_buffer = self.dad.buffer_create(syntax_highlighting)
-        if syntax_highlighting != cons.CUSTOM_COLORS_ID: curr_buffer.begin_not_undoable_action()
+        if syntax_highlighting != cons.RICH_TEXT_ID: curr_buffer.begin_not_undoable_action()
         # loop into rich text, write into the buffer
         child_dom_iter = dom_iter.firstChild
         while child_dom_iter != None:
@@ -145,7 +143,7 @@ class XMLHandler:
             elif child_dom_iter.nodeName == "encoded_image": self.image_deserialize(curr_buffer, child_dom_iter, 1)
             elif child_dom_iter.nodeName == "node": break
             child_dom_iter = child_dom_iter.nextSibling
-        if syntax_highlighting != cons.CUSTOM_COLORS_ID: curr_buffer.end_not_undoable_action()
+        if syntax_highlighting != cons.RICH_TEXT_ID: curr_buffer.end_not_undoable_action()
         curr_buffer.set_modified(False)
         #print unique_id
         # insert the node containing the buffer into the tree
@@ -223,16 +221,21 @@ class XMLHandler:
         if dom_node.hasAttribute(cons.TAG_JUSTIFICATION): justification = dom_node.attributes[cons.TAG_JUSTIFICATION].value
         else: justification = cons.TAG_PROP_LEFT
         if dom_node.hasAttribute("anchor"):
-            pixbuf = gtk.gdk.pixbuf_new_from_file(cons.ANCHOR_CHAR)
-            pixbuf = pixbuf.scale_simple(self.dad.anchor_size, self.dad.anchor_size, gtk.gdk.INTERP_BILINEAR)
+            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(cons.ANCHOR_CHAR, self.dad.anchor_size, self.dad.anchor_size)
             pixbuf.anchor = dom_node.attributes["anchor"].value
+        elif dom_node.hasAttribute("filename"):
+            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(cons.FILE_CHAR, self.dad.embfile_size, self.dad.embfile_size)
+            pixbuf.filename = dom_node.attributes["filename"].value
+            pixbuf.embfile = base64.b64decode(dom_node.firstChild.data)
         else:
             if version == 2: pixbuf = get_pixbuf_from_encoded_buffer(dom_node.firstChild.data)
             else: pixbuf = get_pixbuf_from_png_encoded_string(dom_node.firstChild.data)
-        if pixbuf: self.dad.image_insert(curr_buffer.get_iter_at_offset(char_offset),
-                                         pixbuf,
-                                         image_justification=justification,
-                                         text_buffer=curr_buffer)
+        if pixbuf:
+            pixbuf.link = dom_node.attributes["link"].value if dom_node.hasAttribute("link") else ""
+            self.dad.image_insert(curr_buffer.get_iter_at_offset(char_offset),
+                pixbuf,
+                image_justification=justification,
+                text_buffer=curr_buffer)
 
     def rich_text_deserialize(self, curr_buffer, dom_node):
         """From the XML rich text to the SourceBuffer"""
@@ -315,7 +318,7 @@ class XMLHandler:
         else:
             start_iter = curr_buffer.get_iter_at_offset(sel_range[0])
             end_iter = curr_buffer.get_iter_at_offset(sel_range[1])
-        if programming_language == cons.CUSTOM_COLORS_ID:
+        if programming_language == cons.RICH_TEXT_ID:
             # rich text insert
             curr_iter = start_iter.copy()
             self.rich_text_attributes_update(curr_iter, self.curr_attributes)
@@ -341,8 +344,8 @@ class XMLHandler:
                 for element in pixbuf_table_codebox_vector:
                     if sel_range: element[1][0] -= sel_range[0]
                     if element[0] == "pixbuf": self.pixbuf_element_to_xml(element[1], dom_iter, self.dom)
-                    elif element[0] == "table": self.table_element_to_xml(element[1], dom_iter)
-                    elif element[0] == "codebox": self.codebox_element_to_xml(element[1], dom_iter)
+                    elif element[0] == "table": self.table_element_to_xml(element[1], dom_iter, self.dom)
+                    elif element[0] == "codebox": self.codebox_element_to_xml(element[1], dom_iter, self.dom)
         else:
             # plain text insert
             self.rich_txt_serialize(dom_iter, start_iter, end_iter, self.curr_attributes)
@@ -352,9 +355,9 @@ class XMLHandler:
                 self.append_dom_node(tree_iter, dom_iter, to_disk)
                 tree_iter = self.dad.treestore.iter_next(tree_iter)
 
-    def codebox_element_to_xml(self, element, dom_node):
+    def codebox_element_to_xml(self, element, dom_node, dom):
         """From element [char_offset, codebox, justification] to dom node"""
-        dom_iter = self.dom.createElement("codebox")
+        dom_iter = dom.createElement("codebox")
         dom_iter.setAttribute("char_offset", str(element[0]))
         if element[2] != cons.TAG_PROP_LEFT: dom_iter.setAttribute(cons.TAG_JUSTIFICATION, element[2])
         dom_iter.setAttribute("frame_width", str(element[1]['frame_width']))
@@ -364,24 +367,24 @@ class XMLHandler:
         dom_iter.setAttribute("highlight_brackets", str(element[1]['highlight_brackets']))
         dom_iter.setAttribute("show_line_numbers", str(element[1]['show_line_numbers']))
         dom_node.appendChild(dom_iter)
-        text_iter = self.dom.createTextNode(element[1]['fill_text'])
+        text_iter = dom.createTextNode(element[1]['fill_text'])
         dom_iter.appendChild(text_iter)
 
-    def table_element_to_xml(self, element, dom_node):
+    def table_element_to_xml(self, element, dom_node, dom):
         """From element [char_offset, table, justification] to dom node"""
-        dom_iter = self.dom.createElement("table")
+        dom_iter = dom.createElement("table")
         dom_iter.setAttribute("char_offset", str(element[0]))
         if element[2] != cons.TAG_PROP_LEFT: dom_iter.setAttribute(cons.TAG_JUSTIFICATION, element[2])
         dom_iter.setAttribute("col_min", str(element[1]['col_min']))
         dom_iter.setAttribute("col_max", str(element[1]['col_max']))
         dom_node.appendChild(dom_iter)
         for row in element[1]['matrix']:
-            dom_row = self.dom.createElement("row")
+            dom_row = dom.createElement("row")
             dom_iter.appendChild(dom_row)
             for cell in row:
-                dom_cell = self.dom.createElement("cell")
+                dom_cell = dom.createElement("cell")
                 dom_row.appendChild(dom_cell)
-                text_iter = self.dom.createTextNode(cell)
+                text_iter = dom.createTextNode(cell)
                 dom_cell.appendChild(text_iter)
 
     def pixbuf_element_to_xml(self, element, dom_node, dom):
@@ -389,13 +392,20 @@ class XMLHandler:
         dom_iter = dom.createElement("encoded_png")
         dom_iter.setAttribute("char_offset", str(element[0]))
         if element[2] != cons.TAG_PROP_LEFT: dom_iter.setAttribute(cons.TAG_JUSTIFICATION, element[2])
-        if "anchor" in dir(element[1]):
+        pixbuf_attrs = dir(element[1])
+        if "anchor" in pixbuf_attrs:
             dom_iter.setAttribute("anchor", element[1].anchor)
-            is_anchor_image = True
-        else: is_anchor_image = False
-        dom_node.appendChild(dom_iter)
-        if not is_anchor_image: text_iter = dom.createTextNode(get_encoded_buffer_from_pixbuf(element[1]))
-        else: text_iter = dom.createTextNode("anchor")
+            dom_node.appendChild(dom_iter)
+            text_iter = dom.createTextNode("anchor")
+        elif "filename" in pixbuf_attrs:
+            dom_iter.setAttribute("filename", element[1].filename)
+            dom_node.appendChild(dom_iter)
+            text_iter = dom.createTextNode(base64.b64encode(element[1].embfile))
+        else:
+            if "link" in dir(element[1]) and element[1].link:
+                dom_iter.setAttribute("link", element[1].link)
+            dom_node.appendChild(dom_iter)
+            text_iter = dom.createTextNode(get_encoded_buffer_from_pixbuf(element[1]))
         dom_iter.appendChild(text_iter)
 
     def rich_txt_serialize(self, dom_node, start_iter, end_iter, curr_attributes, change_case="n", dom=None):
@@ -574,6 +584,7 @@ class XMLHandler:
             if offsets:
                 start_iter = text_buffer.get_iter_at_offset(offsets[0])
                 curr_iter = text_buffer.get_iter_at_offset(offsets[1])
+                end_iter = text_buffer.get_end_iter()
             if curr_iter.compare(end_iter) == 0: break
             else:
                 self.rich_text_attributes_update(curr_iter, self.curr_attributes)
@@ -631,8 +642,7 @@ class XMLHandler:
                 start_offset -= 1
                 end_offset -= 1
                 start_iter = text_buffer.get_iter_at_offset(start_offset)
-        pixbuf = gtk.gdk.pixbuf_new_from_file(cons.ANCHOR_CHAR)
-        pixbuf = pixbuf.scale_simple(self.dad.anchor_size, self.dad.anchor_size, gtk.gdk.INTERP_BILINEAR)
+        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(cons.ANCHOR_CHAR, self.dad.anchor_size, self.dad.anchor_size)
         pixbuf.anchor = self.toc_list[-1][0]
         self.dad.image_insert(start_iter, pixbuf)
         self.dad.ctdb_handler.pending_edit_db_node_buff(node_id, force_user_active=True)
@@ -661,7 +671,7 @@ class StateMachine:
         else: curr_iter = text_buffer.get_start_iter()
         while 1:
             anchor = curr_iter.get_child_anchor()
-            if anchor != None:
+            if anchor:
                 anchor_dir = dir(anchor)
                 if "pixbuf" in anchor_dir:
                     if for_print != 1 or not "anchor" in dir(anchor.pixbuf):
@@ -844,6 +854,15 @@ class StateMachine:
                 return self.nodes_vectors[node_id][self.nodes_indexes[node_id]]
         else: return None
 
+    def delete_states(self, node_id):
+        """Delete the states for the given node_id"""
+        del self.nodes_indexes[node_id]
+        del self.nodes_vectors[node_id]
+        del self.nodes_indicators[node_id]
+        if node_id in self.visited_nodes_list:
+            self.visited_nodes_list.remove(node_id)
+            self.visited_nodes_idx = len(self.visited_nodes_list)-1
+
     def update_state(self, node_id):
         """Update the state for the given node_id"""
         curr_index = self.nodes_indexes[node_id]
