@@ -290,17 +290,18 @@ MA 02110-1301, USA.
     dialog.set_authors(["Giuseppe Penone <giuspen@gmail.com>"])
     dialog.set_documenters(["Robert Boudreau <RobtTheB@gmail.com>"])
     dialog.set_artists(["OCAL <http://www.openclipart.org/>", "Zeltak <zeltak@gmail.com>", "Angelo Penone <angelo.penone@gmail.com>"])
-    dialog.set_translator_credits(_("""Chinese Simplified (zh_CN) Channing Wong <mamimoluo@gmail.com>
-Czech (cs) Pavel Fric <fripohled@blogspot.com>
-Dutch (nl) Patrick Vijgeboom <pj.vijgeboom@gmail.com>
-French (fr) Benoît D'Angelo <benoit.dangelo@gmx.fr> (former Ludovic Troisi)
-German (de) Frank Brungräber <calexu@arcor.de> (former Sven Neubauer)
-Italian (it) Giuseppe Penone <giuspen@gmail.com>
-Polish (pl) Marcin Swierczynski <orneo1212@gmail.com>
-Portuguese Brazil (pt_BR) Vinicius Schmidt <viniciussm@rocketmail.com>
-Russian (ru) Andriy Kovtun <kovtunos@yandex.ru>
-Spanish (es) Daniel MC <i.e.betel@gmail.com>
-Ukrainian (uk) Andriy Kovtun <kovtunos@yandex.ru>"""))
+    dialog.set_translator_credits(_("Armenian")+" (hy) Seda Stamboltsyan <sedastam@yandex.com>"+cons.CHAR_NEWLINE+
+_("Chinese Simplified")+" (zh_CN) Channing Wong <mamimoluo@gmail.com>"+cons.CHAR_NEWLINE+
+_("Czech")+" (cs) Pavel Fric <fripohled@blogspot.com>"+cons.CHAR_NEWLINE+
+_("Dutch")+" (nl) Patrick Vijgeboom <pj.vijgeboom@gmail.com>"+cons.CHAR_NEWLINE+
+_("French")+" (fr) Benoît D'Angelo <benoit.dangelo@gmx.fr>"+cons.CHAR_NEWLINE+
+_("German")+" (de) Frank Brungräber <calexu@arcor.de>"+cons.CHAR_NEWLINE+
+_("Italian")+" (it) Giuseppe Penone <giuspen@gmail.com>"+cons.CHAR_NEWLINE+
+_("Polish")+" (pl) Marcin Swierczynski <orneo1212@gmail.com>"+cons.CHAR_NEWLINE+
+_("Portuguese Brazil")+" (pt_BR) Vinicius Schmidt <viniciussm@rocketmail.com>"+cons.CHAR_NEWLINE+
+_("Russian")+" (ru) Andriy Kovtun <kovtunos@yandex.ru>"+cons.CHAR_NEWLINE+
+_("Spanish")+" (es) Daniel MC <i.e.betel@gmail.com>"+cons.CHAR_NEWLINE+
+_("Ukrainian")+" (uk) Andriy Kovtun <kovtunos@yandex.ru>")
     dialog.set_logo(gtk.gdk.pixbuf_new_from_file(os.path.join(cons.GLADE_PATH, "cherrytree.png")))
     dialog.set_title(_("About CherryTree"))
     dialog.set_transient_for(dad.window)
@@ -321,7 +322,7 @@ def dialog_choose_element_in_list(father_win, title, elements_list, column_title
         buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                  gtk.STOCK_OK, gtk.RESPONSE_ACCEPT) )
     dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
-    dialog.set_default_size(400, 200)
+    dialog.set_default_size(400, 300)
     scrolledwindow = gtk.ScrolledWindow()
     scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
     elements_liststore = gtk.ListStore(str)
@@ -451,12 +452,12 @@ def dialog_image_handle(father_win, title, original_pixbuf):
         else:
             # reduced size visible into the dialog
             if img_parms.width > 900:
-                img_parms.width = 900
-                img_parms.height = img_parms.width / img_parms.image_w_h_ration
+                img_parms_width = 900
+                img_parms_height = img_parms_width / img_parms.image_w_h_ration
             else:
-                img_parms.height = 600
-                img_parms.width = img_parms.height * img_parms.image_w_h_ration
-            pixbuf = img_parms.original_pixbuf.scale_simple(int(img_parms.width), int(img_parms.height), gtk.gdk.INTERP_BILINEAR)
+                img_parms_height = 600
+                img_parms_width = img_parms_height * img_parms.image_w_h_ration
+            pixbuf = img_parms.original_pixbuf.scale_simple(int(img_parms_width), int(img_parms_height), gtk.gdk.INTERP_BILINEAR)
         image.set_from_pixbuf(pixbuf)
     def on_button_rotate_90_cw_clicked(*args):
         img_parms.original_pixbuf = img_parms.original_pixbuf.rotate_simple(270)
@@ -483,6 +484,8 @@ def dialog_image_handle(father_win, title, original_pixbuf):
     def on_key_press_imagehandledialog(widget, event):
         keyname = gtk.gdk.keyval_name(event.keyval)
         if keyname == cons.STR_RETURN:
+            spinbutton_width.update()
+            spinbutton_height.update()
             try: dialog.get_widget_for_response(gtk.RESPONSE_ACCEPT).clicked()
             except: print cons.STR_PYGTK_222_REQUIRED
             return True
@@ -745,14 +748,7 @@ def dialog_link_handle(dad, title, sel_tree_iter):
                 if "pixbuf" in dir(anchor) and "anchor" in dir(anchor.pixbuf):
                     anchors_list.append([anchor.pixbuf.anchor])
             if not curr_iter.forward_char(): break
-        iter_insert = dad.curr_buffer.get_iter_at_mark(dad.curr_buffer.get_insert())
-        iter_bound = dad.curr_buffer.get_iter_at_mark(dad.curr_buffer.get_selection_bound())
-        insert_offset = iter_insert.get_offset()
-        bound_offset = iter_bound.get_offset()
         dad.objects_buffer_refresh()
-        dad.curr_buffer.move_mark(dad.curr_buffer.get_insert(), dad.curr_buffer.get_iter_at_offset(insert_offset))
-        dad.curr_buffer.move_mark(dad.curr_buffer.get_selection_bound(), dad.curr_buffer.get_iter_at_offset(bound_offset))
-        dad.sourceview.scroll_to_mark(dad.curr_buffer.get_insert(), 0.3)
         if not anchors_list:
             dialog_info(_("There are No Anchors in the Selected Node"), dialog)
             return
@@ -845,7 +841,7 @@ def dialog_choose_node(father_win, title, treestore, sel_tree_iter):
     dialog.hide()
     return None if response != gtk.RESPONSE_ACCEPT else node_parms.sel_iter
 
-def dialog_selnode_selnodeandsub_alltree(dad, also_selection, also_include_node_name=False, also_new_node_page=False):
+def dialog_selnode_selnodeandsub_alltree(dad, also_selection, also_include_node_name=False, also_new_node_page=False, also_index_in_page=False):
     """Dialog to select between the Selected Node/Selected Node + Subnodes/All Tree"""
     dialog = gtk.Dialog(title=_("Involved Nodes"),
         parent=dad.window,
@@ -871,6 +867,12 @@ def dialog_selnode_selnodeandsub_alltree(dad, also_selection, also_include_node_
         checkbutton_node_name.set_active(dad.last_include_node_name)
         content_area.pack_start(separator_item)
         content_area.pack_start(checkbutton_node_name)
+    if also_index_in_page:
+        separator_item = gtk.HSeparator()
+        checkbutton_index_in_page = gtk.CheckButton(label=_("Links Tree in Every Page"))
+        checkbutton_index_in_page.set_active(dad.last_index_in_page)
+        content_area.pack_start(separator_item)
+        content_area.pack_start(checkbutton_index_in_page)
     if also_new_node_page:
         checkbutton_new_node_page = gtk.CheckButton(label=_("New Node in New Page"))
         checkbutton_new_node_page.set_active(dad.last_new_node_page)
@@ -890,6 +892,8 @@ def dialog_selnode_selnodeandsub_alltree(dad, also_selection, also_include_node_
     else: ret_val = 4
     if also_include_node_name:
         dad.last_include_node_name = checkbutton_node_name.get_active()
+    if also_index_in_page:
+        dad.last_index_in_page = checkbutton_index_in_page.get_active()
     if also_new_node_page:
         dad.last_new_node_page = checkbutton_new_node_page.get_active()
     dialog.destroy()
@@ -905,14 +909,18 @@ def set_bookmarks_menu_items(inst):
     menu_item.show()
     bookmarks_menu.append(menu_item)
     inst.bookmarks_menu_items = [menu_item]
+    bookmarks_to_rm = []
     for node_id_str in inst.bookmarks:
-        if not long(node_id_str) in inst.nodes_names_dict: continue
+        if not long(node_id_str) in inst.nodes_names_dict:
+            bookmarks_to_rm.append(node_id_str)
+            continue
         menu_item = gtk.ImageMenuItem(inst.nodes_names_dict[long(node_id_str)])
-        menu_item.set_image(gtk.image_new_from_stock("Red Cherry", gtk.ICON_SIZE_MENU))
+        menu_item.set_image(gtk.image_new_from_stock("cherry_red", gtk.ICON_SIZE_MENU))
         menu_item.connect("activate", select_bookmark_node, node_id_str, inst)
         menu_item.show()
         bookmarks_menu.append(menu_item)
         inst.bookmarks_menu_items.append(menu_item)
+    for element in bookmarks_to_rm: inst.bookmarks.remove(element)
 
 def set_menu_items_special_chars(inst):
     """Set Special Chars menu items"""
@@ -1019,7 +1027,7 @@ def bookmarks_handle(dad):
     liststore = gtk.ListStore(str, str, str)
     for node_id_str in dad.bookmarks:
         # icon, node name, node id string
-        liststore.append(["Red Cherry", dad.nodes_names_dict[long(node_id_str)], node_id_str])
+        liststore.append(["cherry_red", dad.nodes_names_dict[long(node_id_str)], node_id_str])
     treeview = gtk.TreeView(liststore)
     treeview.set_headers_visible(False)
     treeview.set_reorderable(True)
