@@ -226,6 +226,10 @@ class TablesHandler:
         def on_key_press_tablehandle(widget, event):
             keyname = gtk.gdk.keyval_name(event.keyval)
             if keyname == cons.STR_RETURN:
+                spinbutton_rows.update()
+                spinbutton_columns.update()
+                spinbutton_col_min.update()
+                spinbutton_col_max.update()
                 try: dialog.get_widget_for_response(gtk.RESPONSE_ACCEPT).clicked()
                 except: print cons.STR_PYGTK_222_REQUIRED
                 return True
@@ -252,7 +256,7 @@ class TablesHandler:
                                                filter_pattern="*.csv",
                                                filter_name=_("CSV File"),
                                                parent=self.dad.window)
-        if filename == None: return
+        if not filename: return
         if len(filename) < 4 or filename[-4:] != ".csv": filename += ".csv"
         self.dad.pick_dir = os.path.dirname(filename)
         table_dict = self.dad.state_machine.table_to_dict(self.curr_table_anchor)
@@ -272,9 +276,9 @@ class TablesHandler:
             self.table_insert(iter_insert)
         else:
             filepath = support.dialog_file_select(filter_pattern=["*.csv"],
-                                                  filter_name=_("CSV File"),
-                                                  curr_folder=self.dad.pick_dir,
-                                                  parent=self.dad.window)
+                filter_name=_("CSV File"),
+                curr_folder=self.dad.pick_dir,
+                parent=self.dad.window)
             if filepath != None:
                 self.dad.pick_dir = os.path.dirname(filepath)
                 support.text_file_rm_emptylines(filepath)
@@ -330,7 +334,8 @@ class TablesHandler:
         anchor.table_col_min = table_col_min
         anchor.table_col_max = table_col_max
         anchor.treeview.set_grid_lines(gtk.TREE_VIEW_GRID_LINES_BOTH)
-        anchor.treeview.connect('button-press-event', self.on_mouse_button_clicked_table, anchor)
+        anchor.treeview.connect('button-press-event', self.on_mouse_button_clicked_treeview_table, anchor)
+        anchor.treeview.connect('key_press_event', self.on_key_press_treeview_table, anchor)
         anchor.frame = gtk.Frame()
         anchor.frame.add(anchor.treeview)
         anchor.frame.set_shadow_type(gtk.SHADOW_NONE)
@@ -367,7 +372,17 @@ class TablesHandler:
         self.dad.curr_buffer.delete(iter_insert, iter_bound)
         self.table_insert(iter_insert, table, table_justification)
 
-    def on_key_press_table_cell(self, widget, event, path, model, col_num):
+    def on_table_cell_populate_popup(self, entry, menu):
+        """Table Cell Populate Popup"""
+        self.curr_table_cell = entry
+        self.dad.menu_populate_popup(menu, cons.get_popup_menu_entries_table_cell(self))
+
+    def curr_table_cell_insert_newline(self, *args):
+        cursor_pos = self.curr_table_cell.get_position()
+        self.curr_table_cell.insert_text(cons.CHAR_NEWLINE, cursor_pos)
+        self.curr_table_cell.set_position(cursor_pos+1)
+
+    def on_table_cell_key_press(self, widget, event, path, model, col_num):
         """Catches Table Cell key presses"""
         keyname = gtk.gdk.keyval_name(event.keyval)
         if event.state & gtk.gdk.SHIFT_MASK:
@@ -376,9 +391,11 @@ class TablesHandler:
             pass
         elif event.state & gtk.gdk.CONTROL_MASK:
             if keyname == "period":
-                cursor_pos = widget.get_position()
-                widget.insert_text(cons.CHAR_NEWLINE, cursor_pos)
-                widget.set_position(cursor_pos+1)
+                self.curr_table_cell = widget
+                self.curr_table_cell_insert_newline()
+                return True
+            elif keyname == "comma":
+                return True
         else:
             if keyname in [cons.STR_RETURN, "Up", "Down"]:
                 if model[path][col_num] != widget.get_text():
@@ -397,7 +414,7 @@ class TablesHandler:
                             next_path = tuple(node_path_list)
                             next_iter = model.get_iter(next_path)
                         #next_iter = model.iter_next(model.get_iter(path))
-                        if not next_iter: return
+                        if not next_iter: return False
                         next_path = model.get_path(next_iter)
                         next_col_num = self.dad.table_columns-1
                 else:
@@ -406,21 +423,23 @@ class TablesHandler:
                         next_path = path
                     else:
                         next_iter = model.iter_next(model.get_iter(path))
-                        if not next_iter: return
+                        if not next_iter: return False
                         next_path = model.get_path(next_iter)
                         next_col_num = 0
                 #print "(path, col_num) = (%s, %s)" % (path, col_num)
                 #print "(next_path, next_col_num) = (%s, %s)" % (next_path, next_col_num)
                 next_column = self.curr_table_anchor.treeview.get_columns()[next_col_num]
                 self.curr_table_anchor.treeview.set_cursor_on_cell(next_path,
-                                                                   focus_column=next_column,
-                                                                   focus_cell=next_column.get_cell_renderers()[0],
-                                                                   start_editing=True)
+                    focus_column=next_column,
+                    focus_cell=next_column.get_cell_renderers()[0],
+                    start_editing=True)
+        return False
 
     def on_table_cell_editing_started(self, cell, editable, path, model, col_num):
         """A Table Cell is going to be Edited"""
         if isinstance(editable, gtk.Entry):
-            editable.connect('key_press_event', self.on_key_press_table_cell, path, model, col_num)
+            editable.connect('key_press_event', self.on_table_cell_key_press, path, model, col_num)
+            editable.connect('populate-popup', self.on_table_cell_populate_popup)
 
     def on_table_cell_edited(self, cell, path, new_text, model, col_num):
         """A Table Cell has been Edited"""
@@ -473,6 +492,12 @@ class TablesHandler:
         """All Rows Actions"""
         treeviewselection = self.curr_table_anchor.treeview.get_selection()
         model, iter = treeviewselection.get_selected()
+        if not iter:
+            curr_iter = model.get_iter_first()
+            if not curr_iter: return
+            while curr_iter:
+                iter = curr_iter
+                curr_iter = model.iter_next(curr_iter)
         if action == "delete": model.remove(iter)
         elif action == "add": model.insert_after(iter, [""]*len(self.curr_table_anchor.headers))
         elif action == "move_up":
@@ -549,12 +574,39 @@ class TablesHandler:
         """Sort all the Rows Ascending"""
         self.table_row_action("sort_asc")
 
-    def on_mouse_button_clicked_table(self, widget, event, anchor):
+    def on_key_press_treeview_table(self, widget, event, anchor):
+        """Catches Table key presses"""
+        keyname = gtk.gdk.keyval_name(event.keyval)
+        if event.state & gtk.gdk.CONTROL_MASK:
+            self.curr_table_anchor = anchor
+            self.dad.object_set_selection(self.curr_table_anchor)
+            if keyname == "comma":
+                if event.state & gtk.gdk.MOD1_MASK:
+                    self.table_row_delete()
+                else: self.table_row_add()
+                return True
+            if keyname == "period":
+                if event.state & gtk.gdk.MOD1_MASK:
+                    self.table_row_up()
+                else: self.table_row_down()
+                return True
+        elif keyname == "Menu":
+            self.curr_table_anchor = anchor
+            self.dad.object_set_selection(self.curr_table_anchor)
+            menu_table = gtk.Menu()
+            self.dad.menu_populate_popup(menu_table, cons.get_popup_menu_table(self.dad))
+            menu_table.popup(None, None, None, 3, event.time)
+            return True
+        return False
+
+    def on_mouse_button_clicked_treeview_table(self, widget, event, anchor):
         """Catches mouse buttons clicks"""
         self.curr_table_anchor = anchor
         self.dad.object_set_selection(self.curr_table_anchor)
         if event.button == 3:
-            self.dad.ui.get_widget("/TableMenu").popup(None, None, None, event.button, event.time)
+            menu_table = gtk.Menu()
+            self.dad.menu_populate_popup(menu_table, cons.get_popup_menu_table(self.dad))
+            menu_table.popup(None, None, None, event.button, event.time)
 
 
 class UTF8Recoder:
